Module: One

Defined in:
lib/opennebula/grpc/vm_pb.rb,
lib/opennebula/grpc/vn_pb.rb,
lib/opennebula/grpc/acl_pb.rb,
lib/opennebula/grpc/vdc_pb.rb,
lib/opennebula/grpc/hook_pb.rb,
lib/opennebula/grpc/host_pb.rb,
lib/opennebula/grpc/user_pb.rb,
lib/opennebula/grpc/zone_pb.rb,
lib/opennebula/grpc/group_pb.rb,
lib/opennebula/grpc/image_pb.rb,
lib/opennebula/grpc/shared_pb.rb,
lib/opennebula/grpc/system_pb.rb,
lib/opennebula/grpc/cluster_pb.rb,
lib/opennebula/grpc/vmgroup_pb.rb,
lib/opennebula/grpc/vrouter_pb.rb,
lib/opennebula/grpc/document_pb.rb,
lib/opennebula/grpc/secgroup_pb.rb,
lib/opennebula/grpc/template_pb.rb,
lib/opennebula/grpc/backupjob_pb.rb,
lib/opennebula/grpc/datastore_pb.rb,
lib/opennebula/grpc/vntemplate_pb.rb,
lib/opennebula/grpc/marketplace_pb.rb,
lib/opennebula/grpc/vm_services_pb.rb,
lib/opennebula/grpc/vn_services_pb.rb,
lib/opennebula/grpc/acl_services_pb.rb,
lib/opennebula/grpc/vdc_services_pb.rb,
lib/opennebula/grpc/hook_services_pb.rb,
lib/opennebula/grpc/host_services_pb.rb,
lib/opennebula/grpc/user_services_pb.rb,
lib/opennebula/grpc/zone_services_pb.rb,
lib/opennebula/grpc/group_services_pb.rb,
lib/opennebula/grpc/image_services_pb.rb,
lib/opennebula/grpc/marketplaceapp_pb.rb,
lib/opennebula/grpc/system_services_pb.rb,
lib/opennebula/grpc/cluster_services_pb.rb,
lib/opennebula/grpc/vmgroup_services_pb.rb,
lib/opennebula/grpc/vrouter_services_pb.rb,
lib/opennebula/grpc/document_services_pb.rb,
lib/opennebula/grpc/secgroup_services_pb.rb,
lib/opennebula/grpc/template_services_pb.rb,
lib/opennebula/grpc/backupjob_services_pb.rb,
lib/opennebula/grpc/datastore_services_pb.rb,
lib/opennebula/grpc/vntemplate_services_pb.rb,
lib/opennebula/grpc/marketplace_services_pb.rb,
lib/opennebula/grpc/marketplaceapp_services_pb.rb

Defined Under Namespace

Modules: Acl, Backupjob, Cluster, Datastore, Document, Group, Hook, Host, Image, Market, Marketapp, Secgroup, System, Tmpl, User, Vdc, Vm, Vmgroup, Vn, Vntemplate, Vrouter, Zone

Constant Summary collapse

ResponseID =
::Google::Protobuf::DescriptorPool.generated_pool.lookup("one.ResponseID").msgclass
ResponseXML =
::Google::Protobuf::DescriptorPool.generated_pool.lookup("one.ResponseXML").msgclass