Method: VirtualMachineHelper#wait_timeout

Defined in:
lib/vm_helper.rb

#wait_timeout(action, timeout = 300) ⇒ Object

This method raises an exception if the timeout is reached The exception needs to be handled in the VMM drivers and any process that uses this method



21
22
23
24
25
26
27
28
# File 'lib/vm_helper.rb', line 21

def wait_timeout(action, timeout = 300)
    time_start = Time.now
    until send(action)
        sleep(1)
        condition = (Time.now - time_start).to_i >= timeout
        raise 'Reached deploy timeout' if condition
    end
end